# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-02 06:06
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [('auth', '__first__'), ('contenttypes', '__first__')]

    operations = [
        migrations.CreateModel(
            name='Vote',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('user_id', models.BigIntegerField()),
                ('object_id', models.PositiveIntegerField()),
                ('create_at', models.DateTimeField(auto_now_add=True)),
                ('content_type', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    to='contenttypes.ContentType')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='vote',
            unique_together=set([('user_id', 'content_type', 'object_id')]),
        ),
    ]
