# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2017-01-06 07:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vote', '0002_auto_20161229_1022'),
    ]

    operations = [
        migrations.AddField(
            model_name='vote',
            name='action',
            field=models.PositiveSmallIntegerField(default=0),
        ),
    ]
